@@ -305,7 +305,7 @@ class Desugarer {
                     break;
 
                 case ObjectField::FIELD_ID:
-                    field.expr1 = str(field.id->name);
+                    field.expr1 = str(field.idLocation, field.id->name);
                     field.kind = ObjectField::FIELD_EXPR;
                     break;
 
@@ -624,9 +624,6 @@ class Desugarer {
 
         SuperVars svs = desugarFields(ast, ast->fields, obj_level);
 
-        for (ComprehensionSpec &spec : ast->specs)
-            desugar(spec.expr, obj_level);
-
         AST *field = ast->fields.front().expr1;
         AST *value = ast->fields.front().expr2;
 
@@ -904,10 +901,7 @@ class Desugarer {
         }
     }
 
-    void desugarFile(AST *&ast, std::map<std::string, VmExt> *tlas)
-    {
-        desugar(ast, 0);
-
+    DesugaredObject *stdlibAST(std::string filename) {
         // Now, implement the std library by wrapping in a local construct.
         Tokens tokens = jsonnet_lex("std.jsonnet", STD_CODE);
         AST *std_ast = jsonnet_parse(alloc, tokens);
@@ -938,7 +932,15 @@ class Desugarer {
             }
         }
         fields.emplace_back(
-            ObjectField::HIDDEN, str(U"thisFile"), str(decode_utf8(ast->location.file)));
+            ObjectField::HIDDEN, str(U"thisFile"), str(decode_utf8(filename)));
+        return std_obj;
+    }
+
+    void desugarFile(AST *&ast, std::map<std::string, VmExt> *tlas)
+    {
+        desugar(ast, 0);
+
+        DesugaredObject *std_obj = stdlibAST(ast->location.file);
 
         std::vector<std::string> empty;
         auto line_end_blank = Fodder{{FodderElement::LINE_END, 1, 0, empty}};
@@ -955,7 +957,6 @@ class Desugarer {
             for (const auto &pair : *tlas) {
                 AST *expr;
                 if (pair.second.isCode) {
-                    // Now, implement the std library by wrapping in a local construct.
                     Tokens tokens = jsonnet_lex("tla:" + pair.first, pair.second.data.c_str());
                     expr = jsonnet_parse(alloc, tokens);
                     desugar(expr, 0);
@@ -993,6 +994,11 @@ class Desugarer {
     }
 };
 
+DesugaredObject *makeStdlibAST(Allocator *alloc, std::string filename) {
+    Desugarer desugarer(alloc);
+    return desugarer.stdlibAST(filename);
+}
+
 void jsonnet_desugar(Allocator *alloc, AST *&ast, std::map<std::string, VmExt> *tlas)
 {
     Desugarer desugarer(alloc);
